/*
**++
**  FACILITY:
**      NEWSRTL
**
**  ABSTRACT:
**      Sundry run time support routines used in a number of NEWS modules
**
**  AUTHOR:
**      Geoff Huston
**
**  COPYRIGHT:
**      Copyright  1989
**
**  MODIFICATION HISTORY:
**      V5.8     8-Mar-1989     GIH
**        - Initial version
**	V6.1	 2-Feb-1992	rankin@eql.caltech.edu
**		lint cleanup from gcc -Wall
**	V6.1	 1-Jul-1992	reggers@julian.uwo.ca,ake@dayton.saic.com
**		data parse fix for nntp client
**	V6.1	 11-Aug-1992	mark@infocomm.com
**	  - Added new routine fgetlc which reads a line with continuation
**	    character support (i.e. trailing \), and allows invisible 
**	    imbedded comment lines which CAN NOT be continued.
**	    I chose to provide this extra routine instead of "fixing" fgetl
**	    since fgetl is used in lots of different contexts some of which
**	    probably don't allow "#" prefixed comments.  The problem case
**	    that initially inspired this was a NEWS.SYS file that happened to
**	    have a "\" at the end of a comment line.  This "\" caused the
**	    subsequent line(s) also be commented out since they were 
**	    interpreted as continuations of the comment line.
**	V6.1b3	 1-Sep-1992	rankin@eql.caltech.edu
**	  -	Add file deletion and string manipulation routines
**		in order to be able to eliminate a lot of redundant code.
**	V6.1b6	 9-Mar-1993	rankin@eql.caltech.edu
**	  -	parse_usenet_date() -- expand too-short `pdate' buffer and
**		add missing specifier for year to second sscanf format string.
**	V6.1b6	 12-Mar-1992	mark@infocomm.com
**	  -	tz_convert() - had the "sign" of the timezone offsets incorrect
**		for the RFC822 allowed symbolic timezone names (EST, CST,
**		MST ,etc.)
**--
**/

#if vax11c
#module NEWSRTL "V6.1"
#include lnmdef
#include psldef
#include starlet
#else
#include <lnmdef.h>
#include <psldef.h>
#include <starlet.h>
#endif

#define _NEWSRTL_C

#include "newsinclude.h"
#include "newsextern.h"

int *c$_tmphead = 0;                        /* head of heap store list */

/*
 *  c$alloc_tmp
 *
 *  Allocate temporary storage for use with c$ calls
 */

int *c$alloc_tmp(size)
  int size;
{
  int *c$_tmp;

  c$_tmp = (int *) malloc(size + 4);
  *c$_tmp = (int) c$_tmphead;
  c$_tmphead = c$_tmp;
  return(c$_tmphead + 1);
}

int *c$realloc_tmp(oldptr,size)
  int *oldptr, size;
{
  int *x, *i, fst;

  x = oldptr - 1;
  i = c$_tmphead;
  while (i && (i != x)) i = *(int **)i;
  if (i) {
    fst = (i == c$_tmphead);
    i = realloc(i,size);
    if (fst) c$_tmphead = i;
    return(i+1);
    }
  return(0);
}

/*
 *  c$rfi
 *
 *  Generate a reference to a temp integer value
 */

int *c$rfi(value)
  int value;
{
  int *tmp = c$alloc_tmp(4);

  *tmp = value;
  return(tmp);
}

/*
 *  c$dsc
 *
 * Allocate a string descriptor in the temp arg area
 */

struct dsc$descriptor_s *c$dsc(str)
  char *str;
{
  struct dsc$descriptor_s *tmpdsc;

  tmpdsc = (struct dsc$descriptor_s *) c$alloc_tmp(8);
  tmpdsc->dsc$w_length = strlen(str);
  tmpdsc->dsc$b_dtype = DSC$K_DTYPE_T;
  tmpdsc->dsc$b_class = DSC$K_CLASS_S;
  tmpdsc->dsc$a_pointer = str;
  return(tmpdsc);
}

/*
 *  c$dscl
 *
 * Update the length field in a string descriptor
 */

struct dsc$descriptor_s *c$dscl(d)
  struct dsc$descriptor_s *d;
{
  d->dsc$w_length = strlen(d->dsc$a_pointer);
  return(d);
}

/*
 *  c$cks
 *
 *  Check return status, signal errors, and free the temp argument area
 */

int c$cks(status)
  int status;
{
  int *tmp;

  if (!(status & 1)) lib$signal(status);
  while (c$_tmphead) {
    tmp = (int *) *c$_tmphead;
    free(c$_tmphead);
    c$_tmphead = tmp;
    }
  return(status);
}

/*
 *  fgetl
 *
 *  Read lines from file with '\' continuation character support
 */

char *fgetl(f)
  FILE *f;
{
  static char *rbuffer = 0;
  static int rb_size;

  char inl[IO_SIZE],
       *c;
    
  if (!rbuffer) rbuffer = (char *) malloc(rb_size = IO_SIZE);
  *rbuffer = '\0';
  for (;;) {
    if (fgets(inl,IO_SIZE,f)) {
      if ((strlen(rbuffer) + strlen(inl) + 1) > rb_size)
        rbuffer = (char *) realloc(rbuffer,rb_size += IO_SIZE);
      strcat(rbuffer,inl);
      if ((c = strchr(rbuffer,'\n')) != 0) {
        if (*(c-1) == '\\') *(c-1) = '\0';
        else return(rbuffer);
        }
      }
    else {
      if (*rbuffer) return(rbuffer);
      else return(0);
      }
    }
}

/*
 *  fgetlc
 *
 *  Read lines from file with '\' continuation character support
 */

char *fgetlc(f)
  FILE *f;
{
  static char *rbuffer = 0;
  static int rb_size;

  char inl[IO_SIZE],
       *c;
    
  if (!rbuffer) rbuffer = (char *) malloc(rb_size = IO_SIZE);
  *rbuffer = '\0';
  for (;;) {
    if (fgets(inl,IO_SIZE,f)) {
      chop_str(inl,'#');
      if ((strlen(rbuffer) + strlen(inl) + 1) > rb_size)
        rbuffer = (char *) realloc(rbuffer,rb_size += IO_SIZE);
      strcat(rbuffer,inl);
      if ((c = strchr(rbuffer,'\n')) != 0) {
        if (*(c-1) == '\\') *(c-1) = '\0';
        else return(rbuffer);
        }
      }
    else {
      if (*rbuffer) return(rbuffer);
      else return(0);
      }
    }
}

/*
 *  strip_compress_lower
 *
 *  Strip off leading and trailing blanks, convert all whitespace to a single
 *  space, and convert all alphas to lower case.
 */

void
strip_compress_lower(s)
  char *s;
{
  char *start = s,
       *r = s;

  while (*s) {
    if (isgraph(*s)) 
/* 5-SEP-1992, Martin Winter   (winter@vision.rs.ch)
   The tolower does not convert Umlaute, but the CLI-routines does it.
   Because of that all Umlaute get wrong. We change that to the same
   way as the CLI-routines do.
*/
      if (((*s >= 'A') && (*s <= 'Z')) ||
          ((*s >= '\xC0') && (*s <= '\xCF')) ||
          ((*s >= '\xD1') && (*s <= '\xDD')))
        *r++ = *s - 'A' + 'a';
      else
        *r++ = *s;

    else if ((r > start) && (*(r-1) != ' ')) *r++ = ' ';
    s++;
    }
  if ((r > start) && (*(r-1) == ' ')) --r;
  *r = '\0';
}


void
strip_compress(s)
  char *s;
{
  char *start = s,
       *r = s;

  while (*s) {
    if (isgraph(*s)) *r++ = *s;
    else if ((r > start) && (*(r-1) != ' ')) *r++ = ' ';
    s++;
    }
  if ((r > start) && (*(r-1) == ' ')) --r;
  *r = '\0';
}

/*
 *  blank_strip
 *
 *  Remove all non-print characters from a string.
 */

void
blank_strip(s)
  char *s;
{
  char *r = s;

  --s;
  while (*++s)
    if (isgraph(*s)) *r++ = *s;
  *r = '\0';
}

/*
 *  lower_case
 *
 *  convert a string to lower case
 */

char *lower_case(s)
  char *s;
{
  register char *p = s;

  while (*p) {
#if vax11c
/* 5-SEP-1992, Martin Winter   (winter@vision.rs.ch)
   The tolower does not convert Umlaute, but the CLI-routines does it.
   Because of that all Umlaute get wrong. We change that to the same
   way as the CLI-routines do.
*/
    if (((*p >= 'A') && (*p <= 'Z')) ||
        ((*p >= '\xC0') && (*p <= '\xCF')) ||
        ((*p >= '\xD1') && (*p <= '\xDD')))
      *p = *p - 'A' + 'a';
#else    
    if (*p >= 'A' && *p <= 'Z') *p += 'a' - 'A' ;
#endif
    p++;
    }
  return(s);
}


/*
 *  substrcmp
 *
 *  return 1 if the second arg is a substring of the first arg
 */

int substrcmp(s,sub)
  char *s,
       *sub;
{
  int i, bl = strlen(sub), sl = strlen(s);

  for (i = 0; i <= (sl - bl); ++i)
    if (!strncmp(&s[i],sub,bl)) return(i+1);
  return(0);
}

/*
 *  util_cpy
 *
 *  Block copy
 */

void util_cpy(result,input)
  char *result, *input;
{
  int i = 0, j = 0;

  while (i < SUBJLEN) result[i++] = input[j] ? input [j++] : input[j];
  result[--i] = '\0';
}

/*
 *  util_idcpy
 *
 *  block copy (including padding nulls)
 */

void util_idcpy(result,input)
  char *result,
       *input;
{
  int i = 0, j = 0;

  while (i < IDLEN) result[i++] = input[j] ? input [j++] : input[j];
  result[--i] = '\0';
}
/*
 *  util_fromcpy
 *
 *  block copy (including padding nulls)
 */

void util_fromcpy(result,input)
  char *result,
       *input;
{
  int i = 0, j = 0;

  while (i < FROMLEN) result[i++] = input[j] ? input [j++] : input[j];
  result[--i] = '\0';
}

/*
 *  util_subjcpy
 *
 *  block copy (including padding nulls)
 */

void util_subjcpy(result,input)
  char *result,
       *input;
{
  int i = 0, j = 0;

  while (i < SUBJLEN) result[i++] = input[j] ? input [j++] : input[j];
  result[--i] = '\0';
}

char *quotes(s)
  char *s;
{
  static char qrtn[IO_SIZE];
  char *cp = qrtn, c;

  while (((c = *cp++ = *s++) == '"') ? *cp++ = '"' : c);
  return(qrtn);
}

/*
 *  file_copy
 *
 *  Use RMS to copy a file.
 */

int file_copy(from,to)
  char *from,
       *to;
{
  struct FAB infab,
             outfab;
  struct RAB inrab,
             outrab;
  struct XABDAT inxabdat,
                outxabdat;
  struct XABFHC inxabfhc,
                outxabfhc;
  struct XABRDT inxabrdt,
                outxabrdt;
  struct XABPRO inxabpro,
                outxabpro;
  int status;
  char rec_buff[63*512];
  short rec_size = sizeof(rec_buff);

  infab = cc$rms_fab;
  infab.fab$l_fna = from;
  infab.fab$b_fns = strlen(from);
  infab.fab$b_fac = FAB$M_BIO | FAB$M_GET ;
  infab.fab$l_xab = (char *) &inxabdat;

  inxabdat = cc$rms_xabdat;
  inxabdat.xab$l_nxt = (char *) &inxabfhc;

  inxabfhc = cc$rms_xabfhc;
  inxabfhc.xab$l_nxt = (char *) &inxabrdt;

  inxabrdt = cc$rms_xabrdt;
  inxabrdt.xab$l_nxt = (char *) &inxabpro;

  inxabpro = cc$rms_xabpro;

  inrab = cc$rms_rab;
  inrab.rab$l_fab = &infab;
  inrab.rab$l_bkt = 0;
  inrab.rab$l_ubf = rec_buff;
  inrab.rab$w_usz = rec_size;

  outfab = cc$rms_fab;
  outfab.fab$l_fna = to;
  outfab.fab$b_fns = strlen(to);
  outfab.fab$l_fop = FAB$M_CBT;
  outfab.fab$w_mrs = rec_size;
  outfab.fab$b_fac = FAB$M_BIO | FAB$M_PUT;
  outfab.fab$b_rat = FAB$M_CR;
  outfab.fab$l_xab = (char *) &outxabdat;

  outxabdat = cc$rms_xabdat;
  outxabdat.xab$l_nxt = (char *) &outxabfhc;

  outxabfhc = cc$rms_xabfhc;
  outxabfhc.xab$l_nxt = (char *) &outxabrdt;

  outxabrdt = cc$rms_xabrdt;
  outxabrdt.xab$l_nxt = (char *) &outxabpro;

  outxabpro = cc$rms_xabpro;

  outrab = cc$rms_rab;
  outrab.rab$l_fab = &outfab;
  outrab.rab$l_bkt = 0;
  outrab.rab$l_rbf = rec_buff;

  if (!((status = sys$open(&infab)) & 1)) return(status);

  outfab.fab$l_alq = infab.fab$l_alq;
  outfab.fab$w_deq = infab.fab$w_deq;
  outfab.fab$b_fsz = infab.fab$b_fsz;
  outfab.fab$l_mrn = infab.fab$l_mrn;
  outfab.fab$w_mrs = infab.fab$w_mrs;
  outfab.fab$b_org = infab.fab$b_org;
  outfab.fab$b_rat = infab.fab$b_rat;
  outfab.fab$b_rfm = infab.fab$b_rfm;

  outxabdat.xab$w_rvn = inxabdat.xab$w_rvn + 1;
  outxabfhc.xab$b_rfo = inxabfhc.xab$b_rfo;
  outxabfhc.xab$b_atr = inxabfhc.xab$b_atr;
  outxabfhc.xab$w_lrl = inxabfhc.xab$w_lrl;
  outxabfhc.xab$b_bkz = inxabfhc.xab$b_bkz;
  outxabfhc.xab$b_hsz = inxabfhc.xab$b_hsz;
  outxabfhc.xab$w_mrz = inxabfhc.xab$w_mrz;
  outxabfhc.xab$w_dxq = inxabfhc.xab$w_dxq;
  outxabfhc.xab$l_sbn = 0;
  outxabrdt.xab$w_rvn = inxabrdt.xab$w_rvn + 1;
  outxabpro.xab$w_pro = inxabpro.xab$w_pro;

  if (!((status = sys$connect(&inrab)) & 1)) return(sys$close(&infab),status);
  if (!((status = sys$create(&outfab)) & 1)) return(sys$close(&infab),status);
  if (!((status = sys$connect(&outrab)) & 1)) return(sys$close(&infab),sys$close(&outfab),status);
  do {
    status = sys$read(&inrab);

    if (status & 1) {
      outrab.rab$w_rsz = inrab.rab$w_rsz;
      status = sys$write(&outrab);
      }
    } while (status & 1);
  if (status == RMS$_EOF) status = 1;
  sys$close(&infab);
  sys$close(&outfab);
  return(status);
}

/*
 *  wild_match
 *
 *  String equality routine, including matching the '*' character.
 */

int wild_match(l,p)
    char *l,
         *p;
{
    if (!*l) {
        if (!*p) return(1);
        else if (*p == '*') return(wild_match(l,p+1));
        else return(0);
        }
    if (*p == '*') {
        while (!wild_match(l,p+1)) {
            l++;
            if (!*l) {
                if (!*(p+1)) return(1);
                else return(0);
                }
            }
        return(1);
        }
    if ((*p == '%') || (*p == '?')) return(wild_match(l+1,p+1));
    if (*p == '^') return((*l == *(p+1)) && wild_match(l+1,p+2));
    return((*l == *p) && wild_match(l+1,p+1));
}

/*
 *  cvt_date_val
 *
 *  Convert from VMS date format to unix integer date value
 */

int
cvt_date(vdate)
  int *vdate;
{
  int offset[2] = {0X4BEB4000, 0X007C9567},
      adjtim[2],
      divisor = 10000000,
      udate;

  if (vdate[1] < 0) {
    int now[2],
        then[2],
        len = 2;

    sys$gettim(now);
    lib$subx(now,vdate,then, &len);
    *vdate = *then;
    *(vdate + 1) = *(then + 1);
    }
  lib$subx(vdate, offset, adjtim, c$ac(2));
  lib$ediv(&divisor, adjtim, &udate, vdate);
  return(udate);
}

int cvt_date_val(str)
    char *str;
{
  char locstr[132],
       *l = locstr,
       *p = str;
  time_t ctime;
  struct tm *stm;
  int vdate[2];

  do {
    *l++ = toupper(*p++);
    } while (*p);
  *l = '\0';

  if (!strcmp(locstr,"TODAY")) {
    time(&ctime);
    stm = localtime(&ctime);
    return(ctime - (stm->tm_sec + (stm->tm_min * 60) + (stm->tm_hour * 3600)));
    }
  if (!strcmp(locstr,"YESTERDAY")) {
    time(&ctime);
    stm = localtime(&ctime);
    return(ctime - (stm->tm_sec + (stm->tm_min * 60) + (stm->tm_hour * 3600) + DAY_SECS));
    }
  if (!strcmp(locstr,"TOMORROW")) {
    time(&ctime);
    stm = localtime(&ctime);
    return(ctime - (stm->tm_sec + (stm->tm_min * 60) + (stm->tm_hour * 3600)) + DAY_SECS);
    }
  if (!(sys$bintim(c$dsc(locstr),vdate) & 1)) return(0);
  else return(cvt_date(vdate));
}

/*
 *  parse_usenet_date
 *
 *  Convert a Unix date string to a time value.
 */

static struct zone {
    char *name;
    int offset;
 } zones[] = {
    {"GMT", 0},
    {"UT", 0},
    {"EDT", -4*60*60},
    {"EST", -5*60*60},
    {"CDT", -5*60*60},
    {"CST", -6*60*60},
    {"MDT", -6*60*60},
    {"MST", -7*60*60},
    {"PDT", -7*60*60},
    {"PST", -8*60*60},
    {NULL, 0},
 };
    

static int tz_convert(tz)
char *tz;
{
  struct zone *z = zones;
  int minutes = 0, sign = 1;

  if ( ! *tz ) return (0);  /* assume GMT if nothing given */

  while (z->name)
    if (0 == strcmp(z->name, tz))
      return(z->offset);
    else
      ++z;
  if (*tz == '+')
    ++tz;
  else
    if (*tz == '-') {
      ++tz;
      sign = -1;
      }
  if ((!isdigit(*tz)) ||(strlen(tz) != 4))
    return (gmt_offset); /* Local Timezone Default */
  return(sign*60*(60*((minutes = atoi(tz))/100) + minutes%100));
}

/*
 *  init_tz - determine local timeone and GMT offset from logicals
 */

void init_tz()
{
  char *gotenv, *cp;

  gotenv = news_getenv("NEWS_TIMEZONE",0);
  if (!gotenv) gotenv = news_getenv("PMDF_TIMEZONE",0);
#ifdef MULTINET
  if (!gotenv) gotenv = news_getenv("MULTINET_TIMEZONE",0);
#endif
#ifdef TWG
  if (!gotenv) gotenv = news_getenv("WIN$TIME_ZONE",0);
#endif
#ifdef TCPWARE
   if (!gotenv) gotenv = news_getenv("SMTP_TIMEZONE",0);
#endif
  if (!gotenv) strcpy(news_timezone,"GMT");
  else {
    strcpy(news_timezone,gotenv);
    cp = news_timezone;
    while (*cp) { *cp = toupper(*cp); ++cp; }
    }

  if (gotenv = news_getenv("NEWS_GMT_OFFSET",0)) {
    int sign, hr, mn, se;

    blank_strip(gotenv);
    cp = gotenv;
    if (*cp == '-') {
      cp++;
      sign = -1;
      }
    if (*cp == '+') {
      cp++;
      sign = 1;
      }
    hr = mn = se = 0;
    if (sscanf(cp,"%d:%d:%d",&hr,&mn,&se) > 0) {
      gmt_offset = (60 * 60 * hr) + (60 * mn) + se;
      gmt_offset *= sign;
      }
    }
  else {
    char scratch[256], *cp2;

    cp = strcpy(scratch,news_timezone);
    while (isspace(*cp)) ++cp;
    cp2 = cp;
    while (!isspace(*cp2)) ++cp2;
    *cp2 = '\0';
    gmt_offset = tz_convert(scratch);
    }

}  /*  end of init_tz()  */

int parse_usenet_date(s)
  char *s;
{
  char *cp, mon[80], pdate[30], tz[80];
  int dom = 0, yr = 0, hr = 0, mn = 0, sc = 0, count;

  if (!s || !*s) return(0);
  if ((cp = strchr(s,',')) != 0) s = ++cp;
  while (isspace(*s)) s++;
  *mon = *tz = '\0';
  if (isdigit(*s)) {
    count = sscanf(s,"%d %s %d %d:%d:%d %s",&dom,mon,&yr,&hr,&mn,&sc,tz);
    /* correct if only last two year digits found - reggers@julian.uwo.ca, ake@dayton.saic.com */
    if (yr < 70) yr += 2000;
    if (yr < 100) yr += 1900;
    }
  else count = sscanf(s,"%*s %s %d %d:%d:%d %d %s",mon,&dom,&hr,&mn,&sc,&yr,tz);

  if (!dom || !yr || !*(cp = mon)) return(0);
  if ((dom <= 0) || (dom >= 32)) return(0);
  if ((yr < 1989) || (yr > 2020)) return(0);
  if (strlen(mon) > 10) return(0);
  if ((hr < 0) || (hr > 23)) return(0);
  if ((mn < 0) || (mn > 59)) return(0);
  if ((sc < 0) || (sc > 59)) return(0);
  while (*cp) { *cp = toupper(*cp); ++cp; }
  if (count == 5)
    if ((cp = strchr(s, ':')) != 0) sscanf(cp, ":%d %s", &mn, tz);

  sprintf(pdate,"%d-%s-%d %d:%d:%d",dom,mon,yr,hr,mn,sc);
  return(cvt_date_val(pdate) + gmt_offset - tz_convert(tz));
}

/*
 *  parse_expiry_date
 *
 *  parse a usenet-format date into an expiry period (days)
 */

int parse_expiry_date(s)
  char *s;
{
  int udate, cdate;

  if (!(udate = parse_usenet_date(s))) return(0);
  time((time_t *) &cdate);
  if (udate <= cdate) return(MIN_DAYS);
  udate = ((udate - cdate) / DAY_SECS) + 1;
  if (udate < MIN_DAYS) udate = MIN_DAYS;
  if (udate > MAX_DAYS) udate = MAX_DAYS;
  return(udate);
}

/*
 *  Delete all versions of a file.
 */
void delete_file_versions(fname)
  const char *fname;
{
  while (delete(fname) == 0) continue;
}

/*
 *  Truncate a string at the indicated character.
 */
char *chop_str(s,c)
  char *s;
  int c;
{
  char *p = strchr(s, c);
  if (p) *p = '\0';
  return p;
}

/*
 *  As above, but increment the return value.
 */
char *chop_str_plus(s,c)
  char *s;
  int c;
{
  char *p = strchr(s, c);
  if (p) *p++ = '\0';
  return p;
}


/*
 * news_strncasecmp - this is a mostly a vanilla strncasecmp, except that if
 * n == 0, it compares until it reaches the end of one string (i.e. behaves
 * like strcmp)
 */

int news_strncasecmp(cp1,cp2,n)
  char *cp1,*cp2;
  int n;
{
  register int tmp,i = 0;

  while (cp1[i]) {
    if (!cp2[i]) return(1);
    if ((tmp = _toupper(cp1[i]) - _toupper(cp2[i]))) return(tmp);
    if (++i == n) return(0);
    }
  if (cp2[i]) return(-1);
  return(0);
}


/*
 *  news_getenv - translate a logical name, and return the equivalence string
 *                (return 0 if no translation or an error encountered)
 *                if systflag != 0, follow rules established in NewsSite.H
 *                for secure logical names.
 */

char *news_getenv(lognam,systflag) 
  char *lognam;
  unsigned long int systflag;
{
  static char eqvstr[LNM$C_NAMLENGTH+1];
  char systbl[] = LOG_TABLE, alltbl[] = "LNM$FILE_DEV";
  unsigned char trnmode = MODE;
  unsigned long int attr = LNM$M_CASE_BLIND;
  unsigned short int retlen;
  struct dsc$descriptor_s tabdsc = {0, DSC$K_DTYPE_B, DSC$K_CLASS_S, 0},
                          lnmdsc = {0, DSC$K_DTYPE_B, DSC$K_CLASS_S, 0};
  struct itmlst { unsigned short int buflen;
                  unsigned short int itmcod;
                  unsigned long int  bufadr;
                  unsigned long int  lenadr; }
    trnlst[2] = {  sizeof eqvstr, LNM$_STRING,   eqvstr,  &retlen,
                               0,           0,        0,        0};


  lnmdsc.dsc$w_length = strlen(lognam);
  lnmdsc.dsc$a_pointer = lognam;

  if (systflag) {

    tabdsc.dsc$w_length = sizeof systbl - 1;
    tabdsc.dsc$a_pointer = systbl;

    if (sys$trnlnm(&attr,&tabdsc,&lnmdsc,&trnmode,&trnlst) & 1) {
      eqvstr[retlen] = '\0';
      return eqvstr;
      }
#ifdef SECURE_LOGICALS
    return 0;
#endif
    }

  tabdsc.dsc$w_length = sizeof alltbl - 1;
  tabdsc.dsc$a_pointer = alltbl;
  if (sys$trnlnm(&attr,&tabdsc,&lnmdsc,0,&trnlst) & 1) {
    eqvstr[retlen] = '\0';
    return eqvstr;
    }
  return 0;
}


/*
 *  init_names - set up node name and address based on logicals
 */

void init_names()
{
  char *gotenv;

  *news_node = '\0';
  gotenv = news_getenv("NEWS_NODE",1);
#ifdef MULTINET
  if (!gotenv) gotenv = news_getenv("MULTINET_SMTP_HOST_NAME",1);
  if (!gotenv) gotenv = news_getenv("MULTINET_HOST_NAME",1);
#endif
#ifdef TCPWARE
  if (!gotenv) gotenv = news_getenv("SMTP_FROM_DOMAIN",1);
  if (!gotenv) gotenv = news_getenv("TCPIP_DOMAINNAME",1);
#endif
#ifdef TWG
  if (!gotenv) gotenv = news_getenv("ARPANET_HOST_NAME",1);
#endif
#ifdef UCX
  if (!gotenv) gotenv = news_getenv("UCX$INET_HOST",1);
#endif
  if (!gotenv) gotenv = news_getenv("INTERNET_HOST_NAME",1);
  if (!gotenv) gotenv = news_getenv("SYS$CLUSTER_NODE",1);
  if (!gotenv) gotenv = news_getenv("SYS$NODE",1);

  if (!gotenv) {
    printf("Can't determine node name (NEWS_NODE) - exiting\n");
    exit(1);
    }
  lower_case(strcpy(news_node,gotenv));
  if ((gotenv = strchr(news_node,':')) != 0) *gotenv = '\0';

  *Node_address = '\0';
  gotenv = news_getenv("NEWS_ADDRESS",1);
  if (gotenv) lower_case(strcpy(Node_address,gotenv));
  else strcpy(Node_address,news_node);


  if (gotenv = news_getenv("NEWS_PATHNAME",1))
    lower_case(strcpy(news_pathname,gotenv));
  else strcpy(news_pathname,news_node);

}  /*  end of init_names()  */
